@@ -16,11 +16,17 @@ from itertools import tee
 from six import string_types
 
 import jmespath
+import json
+import base64
+import logging
 from botocore.exceptions import PaginationError
 from botocore.compat import zip
 from botocore.utils import set_value_from_jmespath, merge_dicts
 
 
+log = logging.getLogger(__name__)
+
+
 class PaginatorModel(object):
     def __init__(self, paginator_config):
         self._paginator_config = paginator_config['pagination']
@@ -39,7 +45,6 @@ class PageIterator(object):
                  result_keys, non_aggregate_keys, limit_key, max_items,
                  starting_token, page_size, op_kwargs):
         self._method = method
-        self._op_kwargs = op_kwargs
         self._input_token = input_token
         self._output_token = output_token
         self._more_results = more_results
@@ -64,8 +69,20 @@ class PageIterator(object):
 
     @resume_token.setter
     def resume_token(self, value):
-        if isinstance(value, list):
-            self._resume_token = '___'.join([str(v) for v in value])
+        if not isinstance(value, dict):
+            raise ValueError("Bad starting token: %s" % value)
+
+        if 'boto_truncate_amount' in value:
+            token_keys = sorted(self._input_token + ['boto_truncate_amount'])
+        else:
+            token_keys = sorted(self._input_token)
+        dict_keys = sorted(value.keys())
+
+        if token_keys == dict_keys:
+            self._resume_token = base64.b64encode(
+                json.dumps(value).encode('utf-8')).decode('utf-8')
+        else:
+            raise ValueError("Bad starting token: %s" % value)
 
     @property
     def non_aggregate_part(self):
@@ -74,7 +91,7 @@ class PageIterator(object):
     def __iter__(self):
         current_kwargs = self._op_kwargs
         previous_next_token = None
-        next_token = [None for _ in range(len(self._input_token))]
+        next_token = dict((key, None) for key in self._input_token)
         # The number of items from result_key we've seen so far.
         total_items = 0
         first_request = True
@@ -100,7 +117,7 @@ class PageIterator(object):
             truncate_amount = 0
             if self._max_items is not None:
                 truncate_amount = (total_items + num_current_response) \
-                    - self._max_items
+                                  - self._max_items
             if truncate_amount > 0:
                 self._truncate_response(parsed, primary_result_key,
                                         truncate_amount, starting_truncation,
@@ -111,7 +128,7 @@ class PageIterator(object):
                 yield response
                 total_items += num_current_response
                 next_token = self._get_next_token(parsed)
-                if all(t is None for t in next_token):
+                if all(t is None for t in next_token.values()):
                     break
                 if self._max_items is not None and \
                         total_items == self._max_items:
@@ -182,10 +199,11 @@ class PageIterator(object):
             op_kwargs[self._limit_key] = self._page_size
 
     def _inject_token_into_kwargs(self, op_kwargs, next_token):
-        for name, token in zip(self._input_token, next_token):
-            if token is None or token == 'None':
-                continue
-            op_kwargs[name] = token
+        for name, token in next_token.items():
+            if (token is not None) and (token != 'None'):
+                op_kwargs[name] = token
+            elif name in op_kwargs:
+                del op_kwargs[name]
 
     def _handle_first_request(self, parsed, primary_result_key,
                               starting_truncation):
@@ -243,22 +261,24 @@ class PageIterator(object):
         # However, even though we only kept 1, this is post
         # left truncation so the next starting index should be 2, not 1
         # (left_truncation + amount_to_keep).
-        next_token.append(str(amount_to_keep + starting_truncation))
+        next_token['boto_truncate_amount'] = \
+            amount_to_keep + starting_truncation
         self.resume_token = next_token
 
     def _get_next_token(self, parsed):
         if self._more_results is not None:
             if not self._more_results.search(parsed):
-                return [None]
-        next_tokens = []
-        for token in self._output_token:
-            next_token = token.search(parsed)
+                return {}
+        next_tokens = {}
+        for output_token, input_key in \
+                zip(self._output_token, self._input_token):
+            next_token = output_token.search(parsed)
             # We do not want to include any empty strings as actual tokens.
             # Treat them as None.
             if next_token:
-                next_tokens.append(next_token)
+                next_tokens[input_key] = next_token
             else:
-                next_tokens.append(None)
+                next_tokens[input_key] = None
         return next_tokens
 
     def result_key_iters(self):
@@ -314,6 +334,30 @@ class PageIterator(object):
     def _parse_starting_token(self):
         if self._starting_token is None:
             return None
+
+        # The starting token is a dict passed as a base64 encoded string.
+        next_token = self._starting_token
+        try:
+            next_token = json.loads(
+                base64.b64decode(next_token).decode('utf-8'))
+            index = 0
+            if 'boto_truncate_amount' in next_token:
+                index = next_token.get('boto_truncate_amount')
+                del next_token['boto_truncate_amount']
+        except (ValueError, TypeError):
+            next_token, index = self._parse_starting_token_deprecated()
+        return next_token, index
+
+    def _parse_starting_token_deprecated(self):
+        """
+        This handles parsing of old style starting tokens, and attempts to
+        coerce them into the new style.
+        """
+        log.debug("Attempting to fall back to old starting token parser. For "
+                  "token: %s" % self._starting_token)
+        if self._starting_token is None:
+            return None
+
         parts = self._starting_token.split('___')
         next_token = []
         index = 0
@@ -328,12 +372,26 @@ class PageIterator(object):
                 next_token.append(None)
             else:
                 next_token.append(part)
-        return next_token, index
+        return self._convert_deprecated_starting_token(next_token), index
 
+    def _convert_deprecated_starting_token(self, deprecated_token):
+        """
+        This attempts to convert a deprecated starting token into the new
+        style.
+        """
+        len_deprecated_token = len(deprecated_token)
+        len_input_token = len(self._input_token)
+        if len_deprecated_token > len_input_token:
+            raise ValueError("Bad starting token: %s" % self._starting_token)
+        elif len_deprecated_token < len_input_token:
+            log.debug("Old format starting token does not contain all input "
+                      "tokens. Setting the rest, in order, as None.")
+            for i in range(len_input_token - len_deprecated_token):
+                deprecated_token.append(None)
+        return dict(zip(self._input_token, deprecated_token))
 
 
 class Paginator(object):
-
     PAGE_ITERATOR_CLS = PageIterator
 
     def __init__(self, method, pagination_config):
@@ -414,6 +472,10 @@ class Paginator(object):
             max_items = int(max_items)
         page_size = pagination_config.get('PageSize', None)
         if page_size is not None:
+            if self._pagination_cfg.get('limit_key', None) is None:
+                raise PaginationError(
+                    message="PageSize parameter is not supported for the "
+                            "pagination interface for this operation.")
             page_size = int(page_size)
         return {
             'MaxItems': max_items,
@@ -422,7 +484,6 @@ class Paginator(object):
         }
 
 
-
 class ResultKeyIterator(object):
     """Iterates over the results of paginated responses.
 
@@ -436,6 +497,7 @@ class ResultKeyIterator(object):
         the result key.
 
     """
+
     def __init__(self, pages_iterator, result_key):
         self._pages_iterator = pages_iterator
         self.result_key = result_key
